@@ -22,7 +22,7 @@
  *  `LICENSE' that comes with the fcron source distribution.
  */
 
- /* $Id: job.c,v 1.57 2003/12/25 22:39:55 thib Exp thib $ */
+ /* $Id: job.c,v 1.61 2004/08/12 09:27:04 thib Exp thib $ */
 
 #include "fcron.h"
 
@@ -41,7 +41,7 @@ char env_home[PATH_LEN];
 char env_shell[PATH_LEN];
 #endif
 
-#ifdef CONFIG_FLASK
+#ifdef WITH_SELINUX
 extern char **environ;
 #endif
 
@@ -52,8 +52,9 @@ die_mail_pame(cl_t *cl, int pamerrno, struct passwd *pas, char *str)
 {
     char buf[MAX_MSG];
 
-    strcpy(buf, str);
-    strcat(buf, " for '%s'");
+    strncpy(buf, str, sizeof(buf)-1);
+    strncat(buf, " for '%s'", sizeof(buf)-strlen(buf)-1);
+    buf[sizeof(buf)-1]='\0';
 
     if (is_mail(cl->cl_option)) {
 	FILE *mailf = create_mail(cl, "Could not run fcron job");
@@ -109,12 +110,20 @@ change_user(struct cl_t *cl)
     setenv("SHELL", pas->pw_shell, 1);
 #else
     {
-	strcat( strcpy(env_user, "USER"), "=");
-	putenv( strncat(env_user, pas->pw_name, sizeof(env_user)-6) );
-	strcat( strcpy(env_home, "HOME"), "=");
-	putenv( strncat(env_home, pas->pw_dir, sizeof(env_home)-6) );
-	strcat( strcpy(env_shell, "SHELL"), "=");
-	putenv( strncat(env_shell, pas->pw_shell, sizeof(env_shell)-7) );
+	strcpy(env_user, "USER=");
+	strncat(env_user, pas->pw_name, sizeof(env_user)-5-1);
+	env_user[sizeof(env_user)-1]='\0';
+	putenv( env_user ); 
+
+	strcpy(env_home, "HOME=");
+	strncat(env_home, pas->pw_dir, sizeof(env_home)-5-1);
+	env_home[sizeof(env_home)-1]='\0';
+	putenv( env_home );
+
+	strcpy(env_shell, "SHELL=");
+	strncat(env_shell, pas->pw_shell, sizeof(env_shell)-6-1);
+	env_shell[sizeof(env_shell)-1]='\0';
+	putenv( env_shell );
     }
 #endif /* HAVE_SETENV */
 
@@ -187,12 +196,16 @@ create_mail(cl_t *line, char *subject)
     int mailfd = temp_file(NULL);
     FILE *mailf = fdopen(mailfd, "r+");
     char hostname[USER_NAME_LEN];
+    /* is this a complete mail address ? (ie. with a "@", not only a username) */
+    char complete_adr = 0;
+    int i;
 
     if ( mailf == NULL )
 	die_e("Could not fdopen() mailfd");
 
     /* write mail header */
-    fprintf(mailf, "To: %s", line->cl_file->cf_user);
+    fprintf(mailf, "To: %s", line->cl_mailto);
+
 #ifdef HAVE_GETHOSTNAME
     if (gethostname(hostname, sizeof(hostname)) != 0) {
 	error_e("Could not get hostname");
@@ -201,7 +214,16 @@ create_mail(cl_t *line, char *subject)
     else {
 	/* it is unspecified whether a truncated hostname is NUL-terminated */
 	hostname[USER_NAME_LEN-1] = '\0';
-	fprintf(mailf, "@%s", hostname);
+
+	/* check if mailto is a complete mail address */
+	for ( i = 0 ; line->cl_mailto[i] != '\0' ; i++ ) {
+	    if ( line->cl_mailto[i] == '@' ) {
+		complete_adr = 1;
+		break;
+	    }
+	}
+	if ( ! complete_adr )
+	    fprintf(mailf, "@%s", hostname);
     }
 #else
     hostname[0] = '\0';
@@ -226,6 +248,7 @@ run_job(struct exe_t *exeent)
     pid_t pid;
     cl_t *line = exeent->e_line;
     int pipe_pid_fd[2];
+    int i=0, j;
 
     /* prepare the job execution */
     if ( pipe(pipe_pid_fd) != 0 ) {
@@ -249,8 +272,8 @@ run_job(struct exe_t *exeent)
  	int to_stdout = foreground && is_stdout(line->cl_option);
 	int pipe_fd[2];
 	short int mailpos = 0;	/* 'empty mail file' size */
-#ifdef CONFIG_FLASK
-	int flask_enabled = is_flask_enabled();
+#ifdef WITH_SELINUX
+	int flask_enabled = is_selinux_enabled();
 #endif
  
 	/* */
@@ -352,10 +375,10 @@ run_job(struct exe_t *exeent)
 	    debug("Execing \"%s -c %s\"", curshell, line->cl_shell);
 #endif /* CHECKJOBS */
 
-#ifdef CONFIG_FLASK
-	    if(flask_enabled)
-		execle_secure(shell, line->cl_file->cf_user_sid, shell, "-c", line->cl_shell, NULL, environ);
-	    else
+#ifdef WITH_SELINUX
+	    if(flask_enabled && setexeccon(line->cl_file->cf_user_context) )
+		die_e("Can't set execute context \"%s\".",
+		      line->cl_file->cf_user_context);
 #endif
 	    execl(curshell, curshell, "-c", line->cl_shell, NULL);
 	    /* execl returns only on error */
@@ -415,9 +438,14 @@ run_job(struct exe_t *exeent)
 	line->cl_file->cf_running += 1;
 
 	/* read the pid of the job */
-	if (read(pipe_pid_fd[0], &(exeent->e_job_pid), sizeof(pid_t)) < sizeof(pid_t)) {
-	    error_e("Could not read job pid : setting it to -1");
-	    exeent->e_job_pid = -1;
+	while ( i < sizeof(pid_t) ) {
+	    j = read(pipe_pid_fd[0], (char*)(&(exeent->e_job_pid))+i, sizeof(pid_t));
+	    if ( j <= 0 && errno != EINTR ) {
+		error_e("Could not read job pid : setting it to -1");
+		exeent->e_job_pid = -1;
+		break;
+	    }
+	    i += j;
 	}
 	close(pipe_pid_fd[0]);
     }
@@ -446,7 +474,7 @@ end_job(cl_t *line, int status, FILE *mailf, short mailpos)
     m = (mail_output == 1) ? " (mailing output)" : "";
     if (WIFEXITED(status) && WEXITSTATUS(status) == 0) {
 	if ( ! is_nolog(line->cl_option) )
-	    explain("Job %s terminated%s", line->cl_shell, m);
+	    explain("Job %s completed%s", line->cl_shell, m);
     }
     else if (WIFEXITED(status)) {
 	warn("Job %s terminated (exit status: %d)%s",
